﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectElectricLocomotive.Entities
{
    /// <summary>
    /// Класс-сущность "Локомотив"
    /// </summary>
    public class EntityLocomotive 
    {
        /// <summary>
        /// Скорость
        /// </summary>
        public int Speed { get; private set; }
        /// <summary>
        /// Вес
        /// </summary>
        public double Weight { get; private set; }
        /// <summary>
        /// Основной цвет
        /// </summary>
        public Color BodyColor { get; private set; }
        /// <summary>
        /// Шаг перемещения Локомотива
        /// </summary>
        public double Step => Speed * 100 / Weight;

        /// <summary>
        /// Конструктор сущности
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес локомотива</param>
        /// <param name="bodyColor">Основной цвет</param>
        public EntityLocomotive(int speed, double weight, Color bodyColor)
        {
            Speed = speed;
            Weight = weight;
            BodyColor = bodyColor;


        }

        /// <summary>
        /// Получение строк со значениями свойств объекта класса-сущности
        /// </summary>
        /// <returns></returns>
        public virtual string[] GetStringRepresentation()
        {
            return new[] { nameof(EntityLocomotive), Speed.ToString(), Weight.ToString(), BodyColor.Name };
        }
        /// <summary>
        /// Создание объекта из массива строк
        /// </summary>
        /// <param name="strs"></param>
        /// <returns></returns>
        public static EntityLocomotive? CreateEntityLocomotive(string[] strs)
        {
            if (strs.Length != 4 || strs[0] != nameof(EntityLocomotive))
            {
                return null;
            }
            return new EntityLocomotive(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]));
        }



        internal void setBodyColor(Color bodyColor)
        {
            BodyColor = bodyColor;
        }
    }
}
