﻿using Microsoft.Extensions.Logging;
using ProjectElectricLocomotive.CollectionGenericObjects;
using ProjectElectricLocomotive.Drawnings;
using ProjectElectricLocomotive.Exceptions;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;


namespace ProjectElectricLocomotive;


/// <summary>
/// Форма работы с компанией и ее коллекцией
/// </summary>
public partial class FormLocomotiveCollection : Form
{

    /// <summary>
    /// Хранилище коллекций
    /// </summary>
    private readonly StorageCollection<DrawningLocomotive> _storageCollection;


    /// <summary>
    /// Логер
    /// </summary>
    private readonly ILogger _logger;

    /// <summary>
    /// Компания
    /// </summary>
    private AbstractCompany? _company = null;
    /// <summary>
    /// Конструктор
    /// </summary>
    public FormLocomotiveCollection(ILogger<FormLocomotiveCollection> logger)
    {
        InitializeComponent();
        _storageCollection = new();
        _logger = logger;
        _logger.LogInformation("Форма загрузилась");
    }


    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void comboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }





    /// <summary>
    /// Добавление локомотива
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonAddLocomotive_Click(object sender, EventArgs e)
    {
        FormElectricLocomotiveConfig form = new();
        //TODO передать метод
        form.Show();
        form.AddEvent(SetLocomotive);
        form.Show();
    }

    /// <summary>
    /// Добавление автомобиля в коллекцию
    /// </summary>
    /// <param name="locomotive"></param>
    private void SetLocomotive(DrawningLocomotive? locomotive)
    {
        try
        {
            if (_company == null || locomotive == null)
            {
                return;
            }
            if (_company + locomotive != -1)
            {
                MessageBox.Show("Объект добавлен");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Добавлен объект: " + locomotive.GetDataForSave());
            }
        }
        catch (ObjectNotFoundException) { }
        catch (CollectionOverflowException ex)
        {
            //MessageBox.Show("Не удалось добавить объект");
            MessageBox.Show(ex.Message);
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }

    /// <summary>
    /// Удаление объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonDelLocomotive_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            return;
        }
        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }
        int pos = Convert.ToInt32(maskedTextBox.Text);
        try
        {
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Удален объект по позиции " + pos);
            }
        }
        catch (Exception ex)
        {
            //MessageBox.Show("Не удалось удалить объект");
            MessageBox.Show(ex.Message);
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }

    }

    /// <summary>
    /// Перерисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        DrawningLocomotive? locomotive = null;
        int counter = 100;
        try
        {
            while (locomotive == null)
            {
                locomotive = _company.GetRandomObject();
                counter--;
                if (counter <= 0)
                {
                    break;
                }
            }
            FormlectricLocomotive form = new()
            {
                SetLocomotive = locomotive
            };
            form.ShowDialog();
        }
        catch (Exception ex)
        {
            MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();

    }
    /// <summary>
    /// Обновление списка в listBoxCollection
    /// </summary>
    private void RerfreshListBoxItems()
    {

        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i];
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }

    /// <summary>
    /// Создание компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        ICollectionGenericObjects<DrawningLocomotive>? collection =
        _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            return;
        }
        switch (comboBoxSelectorCompany.Text)
        {
            case "Депо":
                _company = new LocomotiveDepo(pictureBox.Width,
                pictureBox.Height, collection);
                break;
        }
        panelCompanyTools.Enabled = true;
        RerfreshListBoxItems();
    }



    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonCollectionDel_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        try
        {
            if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
            RerfreshListBoxItems();
            _logger.LogInformation("Коллекция: " + listBoxCollection.SelectedItem.ToString() + " удалена");
        }
        catch (Exception ex)
        {
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }

    /// <summary>
    /// Добавление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonCollectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }
        try
        {
            CollectionType collectionType = CollectionType.None;
            if (radioButtonMassive.Checked)
            {
                collectionType = CollectionType.Massive;
            }
            else if (radioButtonList.Checked)
            {
                collectionType = CollectionType.List;
            }
            _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
            RerfreshListBoxItems();
            _logger.LogInformation("Коллекция добавлена " + textBoxCollectionName.Text);
        }
        catch (Exception ex)
        {
            _logger.LogError("Ошибка: {Message}", ex.Message);
        }
    }

    /// <summary>
    /// Обработка нажатия "Сохранение"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>

    private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.SaveData(saveFileDialog.FileName);
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }

        }


    }

    /// <summary>
    /// Обработка нажатия "Загрузка"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>

    private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.LoadData(openFileDialog.FileName);
                MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                RerfreshListBoxItems();
                _logger.LogInformation("Сохранение в файл: {filename}", openFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Не сохранилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);

            }
        }
    }

    /// <summary>
    /// Сортировка по цвету
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonSortByColor_Click(object sender, EventArgs e)
    {
        CompareLocomotives(new DrawningLocomotiveCompareByColor());
    }

    /// <summary>
    /// Сортировка по типу
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonSortByType_Click(object sender, EventArgs e)
    {
        CompareLocomotives(new DrawningLocomotiveCompareByType());
    }

    /// <summary>
    /// Сортировка по сравнителю
    /// </summary>
    /// <param name="comparer">Сравнитель объектов</param>
    private void CompareLocomotives(IComparer<DrawningLocomotive?> comparer)
    {
        if (_company == null)
        {
            return;
        }
        _company.Sort(comparer);
        pictureBox.Image = _company.Show();
    }

}
