using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Serilog;
using System.Security.Cryptography;
using System;
using NLog.Extensions.Logging;

namespace ProjectElectricLocomotive
{
    internal static class Program
    {
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ServiceCollection services = new();
            ConfigureServices(services);
            ApplicationConfiguration.Initialize();
            using ServiceProvider serviceProvider = services.BuildServiceProvider();
            Application.Run(serviceProvider.GetRequiredService<FormLocomotiveCollection>());
        }

        /// <summary>
        ///   DI
        /// </summary>
        /// <param name="services"></param>
        private static void ConfigureServices(ServiceCollection services)
        {
            string[] path = Directory.GetCurrentDirectory().Split('\\');
            string pathNeed = "";
            for (int i = 0; i < path.Length - 3; i++)
            {
                pathNeed += path[i] + "\\";
            }
            services.AddSingleton<FormLocomotiveCollection>().AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                //option.AddNLog("nlog.config");
                option.AddSerilog(new LoggerConfiguration()
                .ReadFrom.Configuration(new ConfigurationBuilder()
                    .AddJsonFile($"{pathNeed}serilog.json").Build())
                .CreateLogger());
            });
        }


    }
}